
// determine if an element can be focused by keyboard (i.e. is part of the document's sequential focus navigation order)

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = rules;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _utilTabindexValue = require('../util/tabindex-value');

var _utilTabindexValue2 = _interopRequireDefault(_utilTabindexValue);

var _focusRelevantRules = require('./focus-relevant.rules');

var _focusRelevantRules2 = _interopRequireDefault(_focusRelevantRules);

var _utilPlatform = require('../util/platform');

var _utilPlatform2 = _interopRequireDefault(_utilPlatform);

var _utilImageMap = require('../util/image-map');

var _isUtil = require('./is.util');

// Internet Explorer 11 considers fieldset, table, td focusable, but not tabbable
// Internet Explorer 11 considers body to have [tabindex=0], but does not allow tabbing to it
var focusableElementsPattern = /^(fieldset|table|td|body)$/;

function rules(_x2) {
  var _arguments = arguments,
      _left;

  var _again = true;

  _function: while (_again) {
    var element = _x2;
    _again = false;
    var except = _arguments.length <= 1 || _arguments[1] === undefined ? {
      flexbox: false,
      scrollable: false,
      shadow: false
    } : _arguments[1];

    if (element === document) {
      element = document.documentElement;
    }

    if (!element || element.nodeType !== Node.ELEMENT_NODE) {
      throw new TypeError('is/tabbable requires an argument of type Element');
    }

    if (_utilPlatform2['default'].is.BLINK && _utilPlatform2['default'].is.ANDROID && _utilPlatform2['default'].majorVersion > 42) {
      // External keyboard support worked fine in CHrome 42, but stopped working in Chrome 45.
      // The on-screen keyboard does not provide a way to focus the next input element (like iOS does).
      // That leaves us with no option to advance focus by keyboard, ergo nothing is tabbable (keyboard focusable).
      return false;
    }

    var nodeName = element.nodeName.toLowerCase();
    var _tabindex = (0, _utilTabindexValue2['default'])(element);
    var tabindex = _tabindex === null ? null : _tabindex >= 0;
    var hasTabbableTabindexOrNone = tabindex !== false;
    var hasTabbableTabindex = _tabindex !== null && _tabindex >= 0;

    // NOTE: Firefox 31 considers [contenteditable] to have [tabindex=-1], but allows tabbing to it
    // fixed in Firefox 40 the latest - https://bugzilla.mozilla.org/show_bug.cgi?id=1185657
    if (element.hasAttribute('contenteditable')) {
      // tabbing can still be disabled by explicitly providing [tabindex="-1"]
      return hasTabbableTabindexOrNone;
    }

    if (focusableElementsPattern.test(nodeName) && tabindex !== true) {
      return false;
    }

    // in Trident and Gecko SVGElement does not know about the tabIndex property
    if (element.tabIndex === undefined) {
      return false;
    }

    if (nodeName === 'audio') {
      if (!element.hasAttribute('controls')) {
        // In Internet Explorer the <audio> element is focusable, but not tabbable, and tabIndex property is wrong
        return false;
      } else if (_utilPlatform2['default'].is.BLINK) {
        // In Chrome <audio controls tabindex="-1"> remains keyboard focusable
        return true;
      }
    }

    if (nodeName === 'video') {
      if (!element.hasAttribute('controls')) {
        if (_utilPlatform2['default'].is.TRIDENT) {
          // In Internet Explorer the <video> element is focusable, but not tabbable, and tabIndex property is wrong
          return false;
        }
      } else if (_utilPlatform2['default'].is.BLINK || _utilPlatform2['default'].is.GECKO) {
        // In Chrome and Firefox <video controls tabindex="-1"> remains keyboard focusable
        return true;
      }
    }

    if (nodeName === 'object') {
      if (_utilPlatform2['default'].is.BLINK || _utilPlatform2['default'].is.WEBKIT) {
        // In all Blink and WebKit based browsers <embed> and <object> are never keyboard focusable, even with tabindex="0" set
        return false;
      }
    }

    if (nodeName === 'iframe') {
      // In Internet Explorer all iframes are only focusable
      // In WebKit, Blink and Gecko iframes may be tabbable depending on content.
      // Since we can't reliably investigate iframe documents because of the
      // SameOriginPolicy, we're declaring everything only focusable.
      return false;
    }

    if (_utilPlatform2['default'].is.WEBKIT && _utilPlatform2['default'].is.IOS && _utilPlatform2['default'].majorVersion < 10) {
      // iOS 8 only considers a hand full of elements tabbable (keyboard focusable)
      // this holds true even with external keyboards
      var potentiallyTabbable = nodeName === 'input' && element.type === 'text' || element.type === 'password' || nodeName === 'select' || nodeName === 'textarea' || element.hasAttribute('contenteditable');

      if (!potentiallyTabbable) {
        var style = window.getComputedStyle(element, null);
        potentiallyTabbable = (0, _isUtil.isUserModifyWritable)(style);
      }

      if (!potentiallyTabbable) {
        return false;
      }
    }

    if (_utilPlatform2['default'].is.GECKO) {
      // Firefox considers scrollable containers keyboard focusable,
      // even though their tabIndex property is -1
      var style = window.getComputedStyle(element, null);
      if ((0, _isUtil.hasCssOverflowScroll)(style)) {
        return hasTabbableTabindexOrNone;
      }
    }

    if (_utilPlatform2['default'].is.TRIDENT) {
      // IE degrades <area> to script focusable, if the image
      // using the <map> has been given tabindex="-1"
      if (nodeName === 'area') {
        var img = (0, _utilImageMap.getImageOfArea)(element);
        if (img && (0, _utilTabindexValue2['default'])(img) < 0) {
          return false;
        }
      }

      var style = window.getComputedStyle(element, null);
      if ((0, _isUtil.isUserModifyWritable)(style)) {
        // prevent being swallowed by the overzealous isScrollableContainer() below
        return element.tabIndex >= 0;
      }

      if (!except.flexbox && (0, _isUtil.hasCssDisplayFlex)(style)) {
        if (_tabindex === null) {
          var _except = { flexbox: true };

          if (!(_left = (0, _focusRelevantRules2['default'])(element, _except))) {
            return _left;
          }

          _arguments = [_x2 = element, _except];
          _again = true;
          except = nodeName = _tabindex = tabindex = hasTabbableTabindexOrNone = hasTabbableTabindex = potentiallyTabbable = style = style = img = style = _except = undefined;
          continue _function;
        }

        return hasTabbableTabindex;
      }

      // IE considers scrollable containers script focusable only,
      // even though their tabIndex property is 0
      if ((0, _isUtil.isScrollableContainer)(element, nodeName)) {
        return false;
      }

      var _parent = element.parentElement;
      if (_parent) {
        var parentNodeName = _parent.nodeName.toLowerCase();
        var parentStyle = window.getComputedStyle(_parent, null);
        // IE considers scrollable bodies script focusable only,
        if ((0, _isUtil.isScrollableContainer)(_parent, nodeName, parentNodeName, parentStyle)) {
          return false;
        }

        // Children of focusable elements with display:flex are focusable in IE10-11,
        // even though their tabIndex property suggests otherwise
        if ((0, _isUtil.hasCssDisplayFlex)(parentStyle)) {
          // value of tabindex takes precedence
          return hasTabbableTabindex;
        }
      }
    }

    // http://www.w3.org/WAI/PF/aria-practices/#focus_tabindex
    return element.tabIndex >= 0;
  }
}

module.exports = exports['default'];
//# sourceMappingURL=tabbable.rules.js.map